#include <bluetooth/bluetooth.h>
//#if (defined (NRF52840_XXAA) || defined (DEVELOP_IN_NRF52840))
#if defined (NRF5340_XXAA_APPLICATION)
  #define CLOCK_FEATURE_HFCLK_DIVIDE_PRESENT
  #include "nrfx_clock.h"
#endif
#include <zephyr.h>
#include <sys/printk.h>
#include <devicetree.h>
#include <logging/log.h>
#include <device.h>
#include <drivers/gpio.h>
#include <hal/nrf_gpio.h>
#include <drivers/gpio.h>
#include <sys/reboot.h>
#include "mcu_flash.h"
#include "dog.h"
#include "pm.h"
#include <settings/settings.h>
#include <mgmt/mcumgr/smp_bt.h>
#include "os_mgmt/os_mgmt.h"
#include "img_mgmt/img_mgmt.h"

LOG_MODULE_REGISTER(main, 4);
//---------------------------------DFU----------------------------------------------------
#define UPDEVICE_NAME             "Hello_Update"
#define UPDEVICE_NAME_LEN         sizeof(UPDEVICE_NAME)

static struct bt_data upad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	BT_DATA(BT_DATA_NAME_COMPLETE, UPDEVICE_NAME, UPDEVICE_NAME_LEN),
};

static const struct bt_data upsd[] = {
	BT_DATA_BYTES(BT_DATA_UUID128_ALL,
				0x84, 0xaa, 0x60, 0x74, 0x52, 0x8a, 0x8b, 0x86,
				0xd3, 0x4c, 0xb7, 0x1d, 0x1d, 0xdc, 0x53, 0x8d),
};
void ota(void);
static bool will_ota=false;
static bool runing=true;
//----------------------------------------------------------------------------------------
static const struct gpio_dt_spec button0 = GPIO_DT_SPEC_GET_OR(DT_ALIAS(sw0), gpios,{0});
static struct gpio_callback button_cb_data1;
void button_pressed(const struct device *dev, struct gpio_callback *cb,uint32_t pins)
{
	gpio_remove_callback(button0.port, &button_cb_data1);
	printk("button_pressed0\n");
	will_ota=true;
	runing=false;
}
void init_buttonled(void)
{
	if (!device_is_ready(button0.port)) {
		printk("Error: button device %s is not ready\n",button0.port->name);
		return;
	}

	int ret = gpio_pin_configure_dt(&button0, GPIO_INPUT);
	if (ret != 0) {
		printk("Error %d: failed to configure %s pin %d\n",ret, button0.port->name, button0.pin);
		return;
	}

	ret = gpio_pin_interrupt_configure_dt(&button0,GPIO_INT_EDGE_TO_ACTIVE);
	if (ret != 0) {
		printk("Error %d: failed to configure interrupt on %s pin %d\n",
			ret, button0.port->name, button0.pin);
		return;
	}

	gpio_init_callback(&button_cb_data1, button_pressed, BIT(button0.pin));
	gpio_add_callback(button0.port, &button_cb_data1);
}

//----------------------------------------------------------------------------------------
void ota(void)
{
	printk("\n----------------OTA doing----------------\n");

	os_mgmt_register_group();
	img_mgmt_register_group();
	smp_bt_register();

	int err = bt_enable(NULL);
	if (err) {
		printk("Bluetooth init failed (err %d)\n", err);
		return;
	}
	if (IS_ENABLED(CONFIG_SETTINGS)) {
		settings_load();
	}

	err = bt_le_adv_start(BT_LE_ADV_CONN, upad, ARRAY_SIZE(upad),upsd, ARRAY_SIZE(upsd));
	if (err) {
		printk("Advertising failed to start (err %d)\n", err);
		return;
	}
	printk("   OTA Advertising started: %s\n",bt_get_name());
	
	// if wait no OTA start, timeout 2min will reboot
	for (int i=0;i<120;i++) {
		k_sleep(K_MSEC(1000));
	}
	printk("\n---------------OTA timeout---------------\n\n");
	sys_reboot(0);
}

void main(void)
{
	struct bt_le_ext_adv *adv=NULL;
	printk("********Hello World!********\n" );

	mcu_flash_init();
	if(mcu_flash_need_ota())
	{
		runing=false;
		mcu_flash_write_ota(false);//clear ota tag
		ota();
	}else 
		printk("Starting Demo \n");
	
	init_buttonled();

	//---------------------do period type bluetooth------------------------
	int err = bt_enable(NULL);
	if (err) {
		printk("Bluetooth init failed (err %d)\n", err);
		return;
	}
	if (IS_ENABLED(CONFIG_SETTINGS)) {
		settings_load();
	}
	bt_le_ext_adv_create(BT_LE_EXT_ADV_NCONN_NAME, NULL, &adv);
	bt_le_per_adv_set_param(adv, BT_LE_PER_ADV_DEFAULT);
	bt_le_per_adv_start(adv);
	bt_le_ext_adv_start(adv, BT_LE_EXT_ADV_START_DEFAULT);
	printk("done.\n");
	//---------------------------------------------------------------------

	init_dog();
	while(runing){
		k_msleep(1000);
		feed_dog();
	}

	printk("---------------------------end-------------------------------\n");
	if(adv)bt_le_per_adv_stop(adv);
	if(adv)bt_le_ext_adv_stop(adv);

	uninit_dog();
	if(will_ota)
	{
		printk("------will reboot for ota\n");
		mcu_flash_write_ota(true);//add ota tag
		sys_reboot(0);
	}
	else
		go_system_off();

}
